#include "config.h"
#include "imapfilter.h"


extern conn_t connpri, connaux;
extern unsigned int options;


#ifdef DEBUG
/*
 * Test/ping server.
 */
int
test(conn_t * conn)
{
	return server_response(conn, imap_noop(conn));
}
#endif


/*
 * Check server's capabilities.
 */
int
check_capabilities(conn_t * conn)
{
	conn->caps = CAPABILITY_NONE;

	return capability_response(conn, imap_capability(conn));
}


/*
 * Get namespace of mail server's mailboxes.
 */
int
check_namespace(conn_t * conn)
{
	conn->nsp.prefix[0] = conn->nsp.delim = '\0';

	if (!(options & OPTION_NAMESPACE) ||
	    !(conn->caps & CAPABILITY_NAMESPACE))
		return 0;
	else
		return namespace_response(conn, imap_namespace(conn));
}


#ifdef SSL_TLS
/*
 * Begin TLS negotiation (STARTTLS).
 */
int
negotiate_tls(conn_t * conn)
{
	int r;

	r = server_response(conn, imap_starttls(conn));
	init_secure_connection(conn, SSL_TLS_V1);

	return r;
}
#endif


/*
 * Login to server.
 */
int
login(conn_t * conn, char *user, char *pass)
{
	return server_response(conn, imap_login(conn, user, pass));
}



/*
 * Check if a mailbox exists.
 */
int
check_mailbox(conn_t * conn, char *mbox)
{
	return server_response(conn, imap_status(conn, mbox, "MESSAGES"));
}


/*
 * Open mailbox in read-write mode.
 */
int
select_mailbox(conn_t * conn, char *mbox)
{
	int r;

	if (mailbox_status(conn, mbox) == -2)
		return -2;	/* No messages exist. No filters need to be
				 * applied. */

	r = select_response(conn, imap_select(conn,
		apply_namespace(mbox, conn->nsp.prefix, conn->nsp.delim)));

	log_info(LOG_MAILBOX, mbox);

	return r;
}


/*
 * Get mailbox's status.
 */
int
mailbox_status(conn_t * conn, char *mbox)
{
	return status_response(conn, imap_status(conn,
		apply_namespace(mbox, conn->nsp.prefix, conn->nsp.delim),
		"MESSAGES RECENT UNSEEN"), mbox);
}


/*
 * Close examined/selected mailbox.
 */
int
close_mailbox(conn_t * conn)
{
	return server_response(conn, imap_close(conn));
}


/*
 * Logout from server.
 */
int
logout(conn_t * conn)
{
	return logout_response(conn, imap_logout(conn));
}
